`timescale 1ns / 1ps
`default_nettype none

//******************************************************************************
//* HDMI ad teszt alkalmazs.                                                 *
//******************************************************************************
module hdmi_tx(
   //rajel s reset.
   input  wire       clk100M,             //100 MHz rajel bemenet.
   input  wire       rstbt,               //Reset bemenet (RST gomb).
   
   //A HDMI kimeneti csatlakoz jelei.
   output wire       hdmi_tx_d0_p,        //TMDS adat 0.
   output wire       hdmi_tx_d0_n,
   output wire       hdmi_tx_d1_p,        //TMDS adat 1.
   output wire       hdmi_tx_d1_n,
   output wire       hdmi_tx_d2_p,        //TMDS adat 2.
   output wire       hdmi_tx_d2_n,
   output wire       hdmi_tx_clk_p,       //TMDS pixel rajel.
   output wire       hdmi_tx_clk_n,
   input  wire       hdmi_tx_cec,         //HDMI CEC (nem hasznlt).
   input  wire       hdmi_tx_hpdn,        //HDMI hot-plug rzkels (nem hasznlt).
   input  wire       hdmi_tx_scl,         //EDID EEPROM I2C interfsz (nem hasznlt).
   input  wire       hdmi_tx_sda
);

//******************************************************************************
//* MMCM pldnyostsa a szksges rajelek ellltshoz:                   *
//* - clk   : 106,47 MHz pixel rajel                                          *
//* - clk_5x: 532,35 MHz rajel a soros adatok kilptetshez                  *
//******************************************************************************
wire mmcm_clkout0, mmcm_clkout1, mmcm_clkfb, mmcm_locked;

MMCME2_ADV #(
   .BANDWIDTH           ("OPTIMIZED"),
   .CLKOUT4_CASCADE     ("FALSE"),
   .COMPENSATION        ("ZHOLD"),
   .STARTUP_WAIT        ("FALSE"),
   .DIVCLK_DIVIDE       (6),              //A globlis oszts rtke.
   .CLKFBOUT_MULT_F     (63.875),         //A globlis szorzs rtke.
   .CLKFBOUT_PHASE      (0.000),
   .CLKFBOUT_USE_FINE_PS("FALSE"),
   .CLKOUT0_DIVIDE_F    (2.0),            //A CLKOUT0 kimenetre belltott oszts.
   .CLKOUT0_PHASE       (0.000),
   .CLKOUT0_DUTY_CYCLE  (0.500),
   .CLKOUT0_USE_FINE_PS ("FALSE"),
   .CLKOUT1_DIVIDE      (10),             //A CLKOUT1 kimenetre belltott oszts.
   .CLKOUT1_PHASE       (0.000),
   .CLKOUT1_DUTY_CYCLE  (0.500),
   .CLKOUT1_USE_FINE_PS ("FALSE"),
   .CLKIN1_PERIOD       (10.000)          //Bemeneti rajel peridusid [ns].
) mmcm_adv_inst (
   //Kimen rajelek.
   .CLKFBOUT            (mmcm_clkfb),
   .CLKOUT0             (mmcm_clkout0),
   .CLKOUT1             (mmcm_clkout1),
   //Bemen rajelek.
   .CLKFBIN             (mmcm_clkfb),
   .CLKIN1              (clk100M),
   .CLKIN2              (1'b0),
   //Az rajel bemenet kivlaszt jele (a CLKIN1 bemenetet hasznljuk).
   .CLKINSEL            (1'b1),
   //A dinamikos tkonfigurlshoz tartoz jelek.
   .DADDR               (7'h0),
   .DCLK                (1'b0),
   .DEN                 (1'b0),
   .DI                  (16'h0),
   .DO                  (),
   .DRDY                (),
   .DWE                 (1'b0),
   //A dinamikos fzis lptetshez tartoz jelek.
   .PSCLK               (1'b0),
   .PSEN                (1'b0),
   .PSINCDEC            (1'b0),
   .PSDONE              (),
   //Egyb vezrl s sttusz jelek.
   .LOCKED              (mmcm_locked),
   .PWRDWN              (1'b0),
   .RST                 (rstbt)
);

//rajel bufferek.
wire clk, clk_5x;

BUFG BUFG_fastclk(.I(mmcm_clkout0), .O(clk_5x));
BUFG BUFG_slowclk(.I(mmcm_clkout1), .O(clk));

//Reset jel.
wire rst = ~mmcm_locked;


//******************************************************************************
//* 1440 x 900 @ 60 Hz VGA idzts genertor.                                 *
//******************************************************************************
wire [10:0] h_cnt;
wire [9:0]  v_cnt;
wire        h_sync, v_sync, blank;

vga_timing vga_timing(
   //rajel s reset.
   .clk(clk),                          //Pixel rajel bemenet.
   .rst(rst),                          //Reset bemenet.
   
   //Az aktulis pixel pozci.
   .h_cnt(h_cnt),                      //X-koordinta.
   .v_cnt(v_cnt),                      //Y-koordinta.
   
   //Szinkron s kiolt jelek.
   .h_sync(h_sync),                    //Horizontlis szinkron pulzus.
   .v_sync(v_sync),                    //Vertiklis szinkron pulzus.
   .blank(blank)                       //Kiolt jel.
);

//A megjelentend minta ellltsa.
wire [7:0] red   = {8{h_cnt[6] ^ v_cnt[6]}};
wire [7:0] green = {8{h_cnt[7] ^ v_cnt[7]}};
wire [7:0] blue  = {8{h_cnt[8] ^ v_cnt[8]}};


//******************************************************************************
//* TMDS ad.                                                                  *
//******************************************************************************
tmds_transmitter tmds_transmitter(
   //rajel s reset.
   .clk(clk),                          //Pixel rajel bemenet.
   .clk_5x(clk_5x),                    //5x pixel rajel bemenet.
   .rst(rst),                          //Reset jel.

   //Bemeneti video adatok.
   .red_in(red),                       //Piros sznkomponens.
   .green_in(green),                   //Zld sznkomponens.
   .blue_in(blue),                     //Kk sznkomponens.
   .blank_in(blank),                   //A nem lthat kptartomny jelzse.
   .hsync_in(h_sync),                  //Horizontlis szinkronjel.
   .vsync_in(v_sync),                  //Vertiklis szinkronjel.
   
   //Kimen TMDS jelek.
   .tmds_data0_out_p(hdmi_tx_d0_p),    //Adat 0.
   .tmds_data0_out_n(hdmi_tx_d0_n),
   .tmds_data1_out_p(hdmi_tx_d1_p),    //Adat 1.
   .tmds_data1_out_n(hdmi_tx_d1_n),
   .tmds_data2_out_p(hdmi_tx_d2_p),    //Adat 2.
   .tmds_data2_out_n(hdmi_tx_d2_n),
   .tmds_clock_out_p(hdmi_tx_clk_p),   //Pixel rajel.
   .tmds_clock_out_n(hdmi_tx_clk_n)
);

endmodule

`default_nettype wire

